#########################
source("./library.R")

########1. quantile
#####metadata: contain library, sample and batch columns
Norm_quantile<-function(exprMat_count,floor_value=1){
library(limma)
quan<-normalizeQuantiles(exprMat_count, ties=FALSE)
logquan<-apply(quan,2,function(x){log2(x+floor_value)})
return(logquan)
}

########2.combat-seq

Norm_combatseq<-function(exprMat_count,batch,group){

library(edgeR)
library(sva)

exprMat_count_adjust<- ComBat_seq(exprMat_count, batch=batch, group=NULL)
ubatch<-unique(as.character(batch))

logCPM<-matrix(0,ncol=ncol(exprMat_count_adjust),nrow=nrow(exprMat_count_adjust))
rownames(logCPM)<-rownames(exprMat_count_adjust)
colnames(logCPM)<-colnames(exprMat_count_adjust)  

for ( i in 1:length(ubatch)){
  
  library <- colnames(exprMat_count)
  exprMat <- exprMat_count_adjust[,match(library,colnames(exprMat_count_adjust))]
    
  dge <- DGEList(counts = exprMat)
  design <- model.matrix(~group)
  dge <- calcNormFactors(dge)
  logcpm_perbatch <- cpm(dge, log=TRUE, prior.count=2)
  logCPM[rownames(logcpm_perbatch),colnames(logcpm_perbatch)]<-logcpm_perbatch
	}
return(logCPM)
}


########3. RUV

Norm_RUVs<-function(exprMat_count,group,batch){

library("RUVSeq")

group <- as.factor(group)
set <- newSeqExpressionSet(as.matrix(exprMat_count),
                           phenoData = data.frame(group, row.names=colnames(exprMat_count)))

differences <- makeGroups(group)
genes<-rownames(exprMat_count)
set3 <- RUVs(set, genes, k=length(unique(batch)), differences)
normedCounts<-normCounts(set3)
logRUVs<-apply(normedCounts,2,function(x){log2(x+1)})
return(logRUVs)
}


#########4. ratio

Norm_ratio<-function(logexprMat,group,batch,ref_samples){

logexprMat_refs<-logexprMat[,match(ref_samples,colnames(logexprMat))]
ref_batch<-as.factor(as.character(batch[match(ref_samples,colnames(logexprMat)]))
logexprMat_refs_m<-t(apply(logexprMat_refs,1,function(x){unlist(tapply(x,ref_batch,mean))}))

ubatch<-unique(batch)

ratio_expr<-matrix(0,ncol=ncol(logexprMat),nrow=nrow(logexprMat))
rownames(ratio_expr)<-rownames(logexprMat)
colnames(ratio_expr)<-colnames(logexprMat)  

for ( i in 1:length(ubatch)){
  name<-as.character(colnames(logexprMat_refs)[batch==ubatch[i]])
  logexpr_batch<-logexprMat[,colnames(logexprMat)%in% name]
  m<-logexprMat_refs_m[,colnames(logexprMat_refs_m)==ubatch[i]]
    
  mat<-apply(logexpr_batch,2,function(x){x-m})
  ratio_expr[rownames(mat),colnames(mat)]<-mat
}
return(ratio_expr)
}




###############5. z score

Norm_z<-function(logexprMat,batch){
  batz<-t(apply(logexprMat,1,function(x){unlist(tapply(x,as.factor(batch,function(xx){(xx-mean(xx,na.rm = T))/sd(xx[!is.na(xx)])}))}))
  colnames(batz)<-sapply(strsplit(colnames(batz),"\\."),function(x){x[2]})
  batz<-batz[,match(colnames(logexprMat),colnames(batz))]
  batz<-batz[apply(batz,1,function(x){length(which(is.na(x)))==0}),]
  return(batz)
}

#######6. harmony

Norm_hormony<-function(logexprMat,batch_mat){
library(harmony)
harmony_normed<- HarmonyMatrix(logexprMat, batch_mat, vars_use="batch",do_pca=FALSE)
return(harmony_normed)
}







